// $Id$

Drupal.behaviors.wysiwygToolbarDesigner = function(context) {
  var workspace = $('#wysiwyg-toolbar-designer');
  var designArea = $('#toolbar-groups');
  var changeNotification = $('#wysiwyg-toolbar-designer div.warning');
  var createGroup = function(item) {
    item.addClass('toolbar-group').sortable({
      revert: true,
      items: '.wysiwyg-button',
      connectWith: '#toolbar-groups .toolbar-group',
      stop: function(event,ui){
        changeNotification.fadeIn();
      }
    });

    item.droppable({
      accept: '.template-button',
      drop: function(event,ui) {
        var button = ui.draggable.clone();
        button.removeClass('template-button').addClass('toolbar-button');
        $(this).append(button).sortable('refresh');
        changeNotification.fadeIn();
      }
    });

    item.find('.remove-group').click(function() {
      if ($(this).parent().find('.wysiwyg-button').length == 0 || confirm(Drupal.t("Do you want to remove this group ?"))) {
        $(this).parent().remove();
        changeNotification.fadeIn();
      }
    });
  };

  var reset = function() {
    $('.toolbar-group',designArea).remove();

    for (var i in Drupal.settings.wysiwyg_admin.toolbar) {
      var buttons = Drupal.settings.wysiwyg_admin.toolbar[i];

      var group = $('.toolbar-group-template',workspace).clone().removeClass('toolbar-group-template');

      for (var j in buttons) {
        var buttonClass = '.wysiwyg-button-' + buttons[j].plugin + '-' + buttons[j].button;
        var button = $(buttonClass,$('#toolbar-available-buttons'));

        if (button.length) {
          button = button.clone();
          button.removeClass('template-button').addClass('toolbar-button');

          group.append(button);
        }
      }

      createGroup(group);
      designArea.append(group);
    }

    changeNotification.fadeOut();
  }

  $('.add-toolbar-group',workspace).click(function(){
    // clone from toolbar template
    var group = $('.toolbar-group-template',workspace).clone().removeClass('toolbar-group-template');
    createGroup(group);

    // append to final
    designArea.append(group).sortable('refresh');
    changeNotification.fadeIn();
  });

  $('#toolbar-available-buttons .wysiwyg-button').addClass('template-button').draggable({
    handle: '.handler',
    helper: 'clone',
    revert: 'invalid',
    addClasses: false
  });

  $('#toolbar-available-buttons').droppable({
    accept: '.toolbar-button',
    drop: function(event, ui) {
      ui.draggable.remove();
      changeNotification.fadeIn();
    }
  });

  $('#toolbar-groups').sortable({
    items: '.toolbar-group',
    handle: '.group-handler',
    stop: function(event, ui) {
      changeNotification.fadeIn();
    }
  });

  /* actions */
  $('#reset-design').click(function() {
    if (!changeNotification.is(':hidden') && confirm(Drupal.t('Do you want to reset the changes ?')))
      reset();
  });

  $('#save-design').click(function() {
    // prepare toolbar settings
    var toolbar = "";

    designArea.find('.toolbar-group').each(function(key,groupDom){
      var group = "";

      $('.wysiwyg-button',groupDom).each(function(key,button){
        var cls = /wysiwyg-button-([^-]+)-([^\s]+)/.exec($(button).attr('class'));
      
        group += cls[1] + "." + cls[2] + "|";
      });

      toolbar += group + "\n";
    });

    var button = $(this);
    var progress = $('.ahah-progress',workspace);
    button.attr('disabled','disabled');
    progress.show();

    $.ajax({
      type: 'POST',
      dataType: 'json',
      url: Drupal.settings.wysiwyg_admin.callbackUrl,
      data: {toolbar: toolbar},
      success: function(data){
        button.removeAttr('disabled');
        changeNotification.fadeOut();
        Drupal.settings.wysiwyg_admin.toolbar = data.toolbar;
      },
      complete: function(request,status) {
        if (status == 'error' || status == 'parsererror') {
          alert("Error !!!");
        }

        progress.hide();
      }
    });
  });

  reset();
};
